/*
 * HNReader for Android, with preloading.
 * You can preload the content of HN Links and read the articles + comments when you're offline.
 * @author Vadim Kr.
 * @copyright (c) 2013 bndr
 * @license http://creativecommons.org/licenses/by-sa/3.0/legalcode
 */
package com.example.hnreader;

import android.app.Activity;
import android.app.ProgressDialog;

import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;


import com.example.hnreader.adapters.MainViewAdapter;
import com.example.hnreader.utility.Database;
import com.example.hnreader.utility.XMLParser;

import java.util.ArrayList;
import java.util.concurrent.ExecutionException;

public class HNreader extends Activity {

    ArrayList<NewsItem> items;
    GridView newsList;
    MainViewAdapter listAdapter;
    Database db;
    Handler mHandler;
    public ProgressDialog dialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_view);
        db = Database.getInstance(getApplicationContext());
        mHandler = new Handler();

        showLoading(false);
        updateNewsList();
        getNewsItemsList(false);
        dialog.dismiss();
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.hnreader, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.action_preload:
                showLoading(false);

                Thread t = new Thread(
                        new Runnable() {
                            public void run() {
                                preload_content();
                                mHandler.post(new Runnable() {
                                    public void run() {
                                        db.insertNewItems(items);
                                        dialog.dismiss();
                                    }
                                });
                            }
                        });
                t.start();
                return true;
            case R.id.action_refresh:
                showLoading(false);
                Thread tr = new Thread(
                        new Runnable() {
                            public void run() {
                                updateNewsList();
                                mHandler.post(new Runnable() {
                                    public void run() {
                                        refresh();
                                        getNewsItemsList(true);
                                        dialog.dismiss();
                                    }
                                });
                            }
                        });
                tr.start();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Get Rss as ArrayList of Items
     *
     * @return
     */
    private ArrayList<NewsItem> getRSS() {
        String s = "";
        try {
            s = new Download(this).execute("https://www.hnsearch.com/bigrss").get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }

        ArrayList<NewsItem> rss = new XMLParser().GetRssArray(s);

        return rss;
    }

    /**
     * Save the content of each item to the database. Takes time to finish.
     *
     * @return
     */
    private ArrayList<NewsItem> preload_content() {

        refresh();

        for (int i = 0; i < items.size(); i++) {
            Log.d("DEBUG", "PRELOADING CONTENT: " + items.get(i).url);
            String article = db.getArticle(items.get(i).url);
            String comments = db.getStringFromArray(db.getComments(items.get(i).comments_link));
            items.get(i).setComments(comments);
            items.get(i).setText(article);
        }
        return items;
    }

    /**
     * Refresh the database and remove all content.
     */
    private void refresh() {
        db.refresh();

    }

    /**
     * Get the NewsList. If user is online, get it from internet,
     * if offline get it from database
     */
    private void updateNewsList() {
        items = (!isOnline()) ? db.getAll() : getRSS();
    }

    /**
     * Update the GridView with new elements (if exist), refresh the onclicklisteners
     *
     * @param update
     */
    private void getNewsItemsList(boolean update) {

        newsList = (GridView) findViewById(R.id.news_list);

        if (!update) {
            listAdapter = new MainViewAdapter(this, R.layout.main_view_list, items);
            newsList.setAdapter(listAdapter);
        } else {
            listAdapter.items = items;
            listAdapter.notifyDataSetChanged();
            newsList.setOnItemClickListener(null);
        }
        newsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, final View view,
                                    int position, long id) {
                Intent i = new Intent(getApplicationContext(), SingleView.class);
                i.putExtra("item", items.get(position));
                startActivity(i);
            }
        });
    }

    /**
     * Show the "loading" dialog
     *
     * @param dismiss
     */
    private void showLoading(boolean dismiss) {
        if (dismiss) {
            dialog.hide();
            return;
        }
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setMessage("Loading..Please wait");
            dialog.setProgress(0);
            dialog.setCancelable(false);
        }
        dialog.show();


    }

    /**
     * Check if user has connection to the internet
     * @return
     */
    public boolean isOnline() {
        ConnectivityManager cm =
                (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        if (netInfo != null && netInfo.isConnectedOrConnecting()) {
            return true;
        }
        return false;
    }

}
